#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>

#define B921600 921600 
#define B460800 460800
#define B230400 230400

int speed_arr[]={B921600, B460800, B230400,B115200,B57600,B38400,B19200,B9600,B4800,B2400,B1200,B300,B200,B150,B134,B110,B75,B50};
int name_arr[]={921600,460800,230400,115200,57600,38400,19200,9600,4800,2400,1200,300,200,150,134,110,75,50};


void set_speed(int fd, int speed)
{
	int i;
	int status;
	struct termios opt;
	
	tcgetattr(fd ,&opt);
	for(i=0; i< sizeof(speed_arr)/sizeof(int); i++)
	{
		if(speed == name_arr[i])
		{
			tcflush(fd,TCIOFLUSH);
			cfsetispeed(&opt,speed_arr[i]);
			cfsetospeed(&opt,speed_arr[i]);
			status=tcsetattr(fd,TCSANOW,&opt);
			if(status != 0)
			{
				perror("set attr failed!\n");
				exit(1);
			}
			break;
		}
		tcflush(fd,TCIOFLUSH);
	}
}

int set_com(int fd,int databits,int stopbits,int parity)
{
	struct termios opt;
	
	if( tcgetattr(fd ,&opt) != 0)
	{		
		perror("get attr failed!\n");
		exit(1);
	}
	opt.c_cflag &= ~CSIZE;
	switch(databits)
	{
		case 7:		
			opt.c_cflag |= CS7;
			break;
		case 8:		
			opt.c_cflag |= CS8;
			break;
		default:
			printf("Unsupported data size\n");
			exit(1);
	}
	
	switch(parity)
	{
		case 'n':
		case 'N':		
			opt.c_cflag &= ~PARENB;
			opt.c_iflag &= ~INPCK;
			break;
		case 'o':
		case 'O':		
			opt.c_cflag |= (PARODD|PARENB);
			opt.c_iflag |= INPCK;
			break;
		case 'e':
		case 'E':		
			opt.c_cflag |= PARENB;
			opt.c_cflag &= ~PARODD;
			opt.c_iflag |= INPCK;
			break;
		default:
			printf("Unsupported parity\n");
			exit(1);
	}
	
	switch(stopbits)
	{
		case 1:		
			opt.c_cflag &= ~CSTOPB;
			break;
		case 2:		
			opt.c_cflag |=  CSTOPB;
			break;
		default:
			printf("Unsupported stop bits\n");
			exit(1);
	}
	
	if (parity != 'n')
	{
		opt.c_iflag |= INPCK;
		opt.c_cc[VTIME] = 150;
		opt.c_cc[VMIN] = 0;
	}
	tcflush(fd,TCIFLUSH);
	if(tcsetattr(fd,TCSANOW,&opt) != 0)
	{
		perror("set attr failed!\n");
		exit(1);
	}
	return 0;		
}

int OpenDev(char* Dev)
{
	int fd = open(Dev,O_RDWR);
	if( -1 == fd)
	{
		perror("open failed!\n");
		exit(1);
	}
	else
		return fd;
}

int main(int argc, char *argv[] )
{
	int fd;
	int nread;
	char buf[256];
	FILE* fp;
	int speed,databit,stopbit;
	char parity;
	int  i;
	if(argc != 7)
	{
		printf("parameter is not right\n");
		exit(1);
	}

	fd=OpenDev(argv[1]);
	speed=atoi(argv[2]);
	if(fd > 0)
		set_speed(fd,speed);

	databit=atoi(argv[3]);
	stopbit=atoi(argv[4]);
	parity=*argv[5];
	if(set_com(fd,databit,stopbit,parity) != 0)
	{
		printf("Set Com Error\n");
		exit(1);
	}
	
   
	fp=fopen(argv[6],"w+");
   	while(1)
  	{
		while( (nread=read(fd,buf,255)) > 0)
		{
			buf[nread]='\0';
            		printf("%s",buf);
			fflush(stdout);
			fputs(buf,fp);
			fflush(fp);
		}
 	}
	fclose(fp);
	close(fd);
}
